
# q-expansion of newform 490.4.e.s, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 490
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[197, [1, 0]], [101, [0, -1]]]
aps_data = [[0, 2], [10, -10], [0, -5], [0, 0], [-53, 53], [-25, 0], [14, -14], [0, -95], [0, -1], [-206, 0], [108, -108], [0, 57], [-243, 0], [434, 0], [0, -231], [-263, 263], [24, -24], [0, 116], [204, -204], [484, 0], [-692, 692], [0, -466], [-228, 0], [0, -362], [-854, 0], [780, -780], [0, -1076], [0, -852], [1972, -1972], [2148, 0], [-2449, 0], [0, -1771], [764, -764], [2356, 0], [0, -2018], [1766, -1766], [753, -753], [0, 484], [1565, 0], [0, -777], [-2347, 2347], [2454, 0], [0, 460], [2260, -2260], [2745, 0], [-348, 348], [2601, 0], [-1904, 0], [-5048, 5048], [0, -1036], [0, -2594], [-5910, 0], [-2969, 2969], [6225, 0], [0, 4732], [-1400, 1400], [-6780, 6780], [0, -2216], [1046, -1046], [3669, 0], [-2882, 2882], [-2697, 0], [8146, 0], [-4604, 4604], [0, -9984], [0, -9754], [0, -6615], [6390, 0], [-3704, 3704], [-5702, 0], [388, -388], [0, 12846], [11389, -11389], [0, -902], [3529, 0], [0, -1643], [1184, -1184], [0, -11382], [0, 1475], [-5366, 5366], [-13265, 0], [-12170, 0], [-3312, 3312], [8504, 0], [0, 9898], [0, -1680], [-5875, 0], [0, 6950], [13402, 0], [4551, 0], [0, -2238], [-15616, 15616], [-9064, 9064], [5124, 0], [0, -5556], [7280, 0], [0, -19866], [-327, 327], [0, -1148], [0, 15016], [-22188, 0], [2991, -2991], [4274, -4274], [0, -845], [20108, -20108], [22828, -22828], [-16452, 0], [0, 25188], [-26382, 26382], [-9538, 0], [0, 20191], [-18675, 18675], [12602, 0], [3521, -3521], [-13048, 0], [-7555, 7555], [-31802, 0], [-831, 831], [0, -26441], [19536, 0], [9356, -9356], [24774, 0], [0, -10391], [-15492, 15492], [0, -17064], [-24174, 0], [0, -7078], [0, 31844], [-12143, 0], [0, -2333], [-15749, 15749], [-31797, 0], [0, -2638], [-5054, 0], [0, 21301], [12041, 0], [-11285, 11285], [-8326, 8326], [-4794, 0], [-11375, 11375], [23063, 0], [0, 22058], [2148, -2148], [-24506, 0], [32766, -32766], [-13694, 0], [-14179, 0], [8054, -8054], [0, -26888], [0, -37133], [0, 16747], [49015, 0], [22712, 0], [0, 28448], [39230, -39230], [40982, 0], [0, -43330], [0, 42049], [-19642, 0], [-14088, 14088], [0, 8966], [23132, 0], [30856, 0], [0, 5677], [32238, -32238], [-44613, 44613], [-35812, 35812], [2086, -2086]]
